import socket

import pytest

pytest_plugins = "pytester"


unix_sockets_only = pytest.mark.skipif(
    not hasattr(socket, "AF_UNIX"),
    reason="Skip any platform that does not support AF_UNIX",
)



import pytest

from conftest import unix_sockets_only
from tests.common import assert_socket_blocked

PYFILE_SOCKET_USED_IN_TEST_ARGS = """
    import socket

    def test_socket():
        socket.socket(socket.AF_INET, socket.SOCK_STREAM)
"""


PYFILE_SOCKET_USED_IN_TEST_KWARGS = """
    import socket

    def test_socket():
        socket.socket(family=socket.AF_INET, type=socket.SOCK_STREAM)
"""


PYFILE_SOCKET_NO_ARGS = """
    import socket

    def test_socket():
        socket.socket()
"""


@pytest.mark.parametrize(
    "pyfile",
    [
        PYFILE_SOCKET_USED_IN_TEST_ARGS,
        PYFILE_SOCKET_USED_IN_TEST_KWARGS,
        PYFILE_SOCKET_NO_ARGS,
    ],
)
def test_socket_enabled_by_default(testdir, pyfile):
    testdir.makepyfile(pyfile)
    result = testdir.runpytest()
    result.assert_outcomes(1, 0, 0)
    with pytest.raises(BaseException):
        assert_socket_blocked(result)


def test_global_disable_via_fixture(testdir):
    testdir.makepyfile(
        """
        import pytest
        import pytest_socket
        import socket

        @pytest.fixture(autouse=True)
        def disable_socket_for_all():
            pytest_socket.disable_socket()

        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


@pytest.mark.parametrize(
    "pyfile",
    [
        PYFILE_SOCKET_USED_IN_TEST_ARGS,
        PYFILE_SOCKET_USED_IN_TEST_KWARGS,
        PYFILE_SOCKET_NO_ARGS,
    ],
)
def test_global_disable_via_cli_flag(testdir, pyfile):
    testdir.makepyfile(pyfile)
    result = testdir.runpytest("--disable-socket")
    assert_socket_blocked(result)


def test_help_message(testdir):
    result = testdir.runpytest(
        "--help",
    )
    result.stdout.fnmatch_lines(
        [
            "socket:",
            "*--disable-socket*Disable socket.socket by default to block network*",
        ]
    )


@pytest.mark.parametrize(
    "pyfile",
    [
        PYFILE_SOCKET_USED_IN_TEST_ARGS,
        PYFILE_SOCKET_USED_IN_TEST_KWARGS,
        PYFILE_SOCKET_NO_ARGS,
    ],
)
def test_global_disable_via_config(testdir, pyfile):
    testdir.makepyfile(pyfile)
    testdir.makeini(
        """
        [pytest]
        addopts = --disable-socket
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


def test_disable_socket_marker(testdir):
    testdir.makepyfile(
        """
        import pytest
        import socket

        @pytest.mark.disable_socket
        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


def test_enable_socket_marker(testdir):
    testdir.makepyfile(
        """
        import pytest
        import socket

        @pytest.mark.enable_socket
        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    result.assert_outcomes(1, 0, 0)
    with pytest.raises(BaseException):
        assert_socket_blocked(result)


def test_urllib_succeeds_by_default(testdir):
    testdir.makepyfile(
        """
        try:
            from urllib.request import urlopen
        except ImportError:
            from urllib2 import urlopen

        def test_disable_socket_urllib():
            assert urlopen('https://httpstat.us/200').getcode() == 200
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(1, 0, 0)
    with pytest.raises(BaseException):
        assert_socket_blocked(result)


def test_enabled_urllib_succeeds(testdir):
    testdir.makepyfile(
        """
        import pytest
        import pytest_socket
        try:
            from urllib.request import urlopen
        except ImportError:
            from urllib2 import urlopen

        @pytest.mark.enable_socket
        def test_disable_socket_urllib():
            assert urlopen('https://httpstat.us/200').getcode() == 200
        """
    )
    result = testdir.runpytest("--disable-socket")
    result.assert_outcomes(1, 0, 0)
    with pytest.raises(BaseException):
        assert_socket_blocked(result)


def test_disabled_urllib_fails(testdir):
    testdir.makepyfile(
        """
        import pytest
        try:
            from urllib.request import urlopen
        except ImportError:
            from urllib2 import urlopen

        @pytest.mark.disable_socket
        def test_disable_socket_urllib():
            assert urlopen('https://httpstat.us/200').getcode() == 200
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


def test_double_call_does_nothing(testdir):
    testdir.makepyfile(
        """
        import pytest
        import pytest_socket
        import socket

        def test_double_enabled():
            pytest_socket.enable_socket()
            pytest_socket.enable_socket()
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        def test_double_disabled():
            pytest_socket.disable_socket()
            pytest_socket.disable_socket()
            with pytest.raises(pytest_socket.SocketBlockedError):
                socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        def test_disable_enable():
            pytest_socket.disable_socket()
            pytest_socket.enable_socket()
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(3, 0, 0)
    with pytest.raises(BaseException):
        assert_socket_blocked(result)


def test_socket_enabled_fixture(testdir, socket_enabled):
    testdir.makepyfile(
        """
        import socket
        def test_socket_enabled(socket_enabled):
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(1, 0, 0)
    with pytest.raises(BaseException):
        assert_socket_blocked(result)


def test_mix_and_match(testdir, socket_enabled):
    testdir.makepyfile(
        """
        import socket

        def test_socket1():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        def test_socket_enabled(socket_enabled):
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        def test_socket2():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    result.assert_outcomes(1, 0, 2)


def test_socket_subclass_is_still_blocked(testdir):
    testdir.makepyfile(
        """
        import pytest
        import pytest_socket
        import socket

        @pytest.mark.disable_socket
        def test_subclass_is_still_blocked():

            class MySocket(socket.socket):
                pass

            MySocket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


@unix_sockets_only
def test_unix_domain_sockets_blocked_with_disable_socket(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_unix_socket():
            socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    assert_socket_blocked(result)


@unix_sockets_only
def test_enabling_unix_domain_sockets_with_disable_socket(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_inet():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        def test_unix_socket():
            socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket", "--allow-unix-socket")
    result.assert_outcomes(passed=1, skipped=0, failed=1)



def test_function_with_doctest(testdir):
    testdir.makepyfile(
        '''
        def my_sum(a, b):
            """Sum two values.

            >>> my_sum(1, 1)
            2
            """
            return a + b
        '''
    )
    result = testdir.runpytest("--doctest-modules")
    result.assert_outcomes(passed=1)



import ipaddress
import itertools
import socket
import typing
from collections import defaultdict

import pytest

_true_socket = socket.socket
_true_connect = socket.socket.connect


class SocketBlockedError(RuntimeError):
    def __init__(self, *_args, **_kwargs):
        super().__init__("A test tried to use socket.socket.")


class SocketConnectBlockedError(RuntimeError):
    def __init__(self, allowed, host, *_args, **_kwargs):
        if allowed:
            allowed = ",".join(allowed)
        super().__init__(
            "A test tried to use socket.socket.connect() "
            f'with host "{host}" (allowed: "{allowed}").'
        )


def pytest_addoption(parser):
    group = parser.getgroup("socket")
    group.addoption(
        "--disable-socket",
        action="store_true",
        dest="disable_socket",
        help="Disable socket.socket by default to block network calls.",
    )
    group.addoption(
        "--force-enable-socket",
        action="store_true",
        dest="force_enable_socket",
        help="Force enable socket.socket network calls (override --disable-socket).",
    )
    group.addoption(
        "--allow-hosts",
        dest="allow_hosts",
        metavar="ALLOWED_HOSTS_CSV",
        help="Only allow specified hosts through socket.socket.connect((host, port)).",
    )
    group.addoption(
        "--allow-unix-socket",
        action="store_true",
        dest="allow_unix_socket",
        help="Allow calls if they are to Unix domain sockets",
    )


@pytest.fixture
def socket_disabled(pytestconfig):
    """disable socket.socket for duration of this test function"""
    disable_socket(allow_unix_socket=pytestconfig.__socket_allow_unix_socket)
    yield


@pytest.fixture
def socket_enabled(pytestconfig):
    """enable socket.socket for duration of this test function"""
    enable_socket()
    yield


def _is_unix_socket(family) -> bool:
    try:
        is_unix_socket = family == socket.AF_UNIX
    except AttributeError:
        # AF_UNIX not supported on Windows https://bugs.python.org/issue33408
        is_unix_socket = False
    return is_unix_socket


def disable_socket(allow_unix_socket=False):
    """disable socket.socket to disable the Internet. useful in testing."""

    class GuardedSocket(socket.socket):
        """socket guard to disable socket creation (from pytest-socket)"""

        def __new__(cls, family=-1, type=-1, proto=-1, fileno=None):
            if _is_unix_socket(family) and allow_unix_socket:
                return super().__new__(cls, family, type, proto, fileno)

            raise SocketBlockedError()

    socket.socket = GuardedSocket


def enable_socket():
    """re-enable socket.socket to enable the Internet. useful in testing."""
    socket.socket = _true_socket


def pytest_configure(config):
    config.addinivalue_line(
        "markers", "disable_socket(): Disable socket connections for a specific test"
    )
    config.addinivalue_line(
        "markers", "enable_socket(): Enable socket connections for a specific test"
    )
    config.addinivalue_line(
        "markers",
        "allow_hosts([hosts]): Restrict socket connection to defined list of hosts",
    )

    # Store the global configs in the `pytest.Config` object.
    config.__socket_force_enabled = config.getoption("--force-enable-socket")
    config.__socket_disabled = config.getoption("--disable-socket")
    config.__socket_allow_unix_socket = config.getoption("--allow-unix-socket")
    config.__socket_allow_hosts = config.getoption("--allow-hosts")


def pytest_runtest_setup(item) -> None:
    """During each test item's setup phase,
    choose the behavior based on the configurations supplied.

    This is the bulk of the logic for the plugin.
    As the logic can be extensive, this method is allowed complexity.
    It may be refactored in the future to be more readable.

    If the given item is not a function test (i.e a DoctestItem)
    or otherwise has no support for fixtures, skip it.
    """
    if not hasattr(item, "fixturenames"):
        return

    # If test has the `enable_socket` marker, fixture or
    # it's forced from the CLI, we accept this as most explicit.
    if (
        "socket_enabled" in item.fixturenames
        or item.get_closest_marker("enable_socket")
        or item.config.__socket_force_enabled
    ):
        enable_socket()
        return

    # If the test has the `disable_socket` marker, it's explicitly disabled.
    if "socket_disabled" in item.fixturenames or item.get_closest_marker(
        "disable_socket"
    ):
        disable_socket(item.config.__socket_allow_unix_socket)
        return

    # Resolve `allow_hosts` behaviors.
    hosts = _resolve_allow_hosts(item)

    # Finally, check the global config and disable socket if needed.
    if item.config.__socket_disabled and not hosts:
        disable_socket(item.config.__socket_allow_unix_socket)


def _resolve_allow_hosts(item):
    """Resolve `allow_hosts` behaviors."""
    mark_restrictions = item.get_closest_marker("allow_hosts")
    cli_restrictions = item.config.__socket_allow_hosts
    hosts = None
    if mark_restrictions:
        hosts = mark_restrictions.args[0]
    elif cli_restrictions:
        hosts = cli_restrictions
    socket_allow_hosts(hosts, allow_unix_socket=item.config.__socket_allow_unix_socket)
    return hosts


def pytest_runtest_teardown():
    _remove_restrictions()


def host_from_address(address):
    host = address[0]
    if isinstance(host, str):
        return host


def host_from_connect_args(args):
    address = args[0]

    if isinstance(address, tuple):
        return host_from_address(address)


def is_ipaddress(address: str) -> bool:
    """
    Determine if the address is a valid IPv4 or IPv6 address.
    """
    try:
        ipaddress.ip_address(address)
        return True
    except ValueError:
        return False


def resolve_hostnames(hostname: str) -> typing.Set[str]:
    try:
        return {
            addr_struct[0] for *_, addr_struct in socket.getaddrinfo(hostname, None)
        }
    except socket.gaierror:
        return set()


def normalize_allowed_hosts(
    allowed_hosts: typing.List[str],
) -> typing.Dict[str, typing.Set[str]]:
    """Map all items in `allowed_hosts` to IP addresses."""
    ip_hosts = defaultdict(set)
    for host in allowed_hosts:
        host = host.strip()
        if is_ipaddress(host):
            ip_hosts[host].add(host)
        else:
            ip_hosts[host].update(resolve_hostnames(host))

    return ip_hosts


def socket_allow_hosts(allowed=None, allow_unix_socket=False):
    """disable socket.socket.connect() to disable the Internet. useful in testing."""
    if isinstance(allowed, str):
        allowed = allowed.split(",")

    if not isinstance(allowed, list):
        return

    allowed_ip_hosts_by_host = normalize_allowed_hosts(allowed)
    allowed_ip_hosts_and_hostnames = set(
        itertools.chain(*allowed_ip_hosts_by_host.values())
    ) | set(allowed_ip_hosts_by_host.keys())
    allowed_list = sorted(
        [
            (
                host
                if len(normalized) == 1 and next(iter(normalized)) == host
                else f"{host} ({','.join(sorted(normalized))})"
            )
            for host, normalized in allowed_ip_hosts_by_host.items()
        ]
    )

    def guarded_connect(inst, *args):
        host = host_from_connect_args(args)
        if host in allowed_ip_hosts_and_hostnames or (
            _is_unix_socket(inst.family) and allow_unix_socket
        ):
            return _true_connect(inst, *args)

        raise SocketConnectBlockedError(allowed_list, host)

    socket.socket.connect = guarded_connect


def _remove_restrictions():
    """restore socket.socket.* to allow access to the Internet. useful in testing."""
    socket.socket = _true_socket
    socket.socket.connect = _true_connect



"""Test module to express precedence tests between the different
configuration combinations"""

from tests.common import assert_socket_blocked


def test_disable_via_fixture(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_socket(socket_disabled):
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


def test_disable_via_marker(testdir):
    testdir.makepyfile(
        """
        import socket
        import pytest

        @pytest.mark.disable_socket
        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


def test_global_disable_via_cli_flag(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    assert_socket_blocked(result)


def test_force_enable_socket_via_cli_flag(testdir):
    testdir.makepyfile(
        """
        import socket
        import pytest

        @pytest.mark.disable_socket
        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--force-enable-socket")
    result.assert_outcomes(passed=1)


def test_force_enable_cli_flag_precedence(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket", "--force-enable-socket")
    result.assert_outcomes(passed=1)


def test_global_disable_via_config(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    testdir.makeini(
        """
        [pytest]
        addopts = --disable-socket
        """
    )
    result = testdir.runpytest()
    assert_socket_blocked(result)


def test_enable_via_fixture(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_socket(socket_enabled):
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        def test_socket_disabled():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    assert_socket_blocked(result, passed=1, failed=1)


def test_enable_via_marker(testdir):
    testdir.makepyfile(
        """
        import socket
        import pytest

        @pytest.mark.enable_socket
        def test_socket():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        def test_socket_disabled():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    assert_socket_blocked(result, passed=1, failed=1)


def test_enable_marker_for_test_class(testdir):
    testdir.makepyfile(
        """
        import socket
        import pytest

        @pytest.mark.enable_socket
        class TestSocket:
            def test_socket(self):
                socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        def test_socket_disabled():
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        """
    )
    result = testdir.runpytest("--disable-socket")
    assert_socket_blocked(result, passed=1, failed=1)


def test_global_disable_and_allow_host(testdir, httpbin):
    """Disable socket globally, but allow a specific host"""
    testdir.makepyfile(
        f"""
        from urllib.request import urlopen

        def test_urlopen():
            assert urlopen("{httpbin.url}/")

        def test_urlopen_disabled():
            assert urlopen("https://google.com/")
        """
    )
    result = testdir.runpytest("--disable-socket", f"--allow-hosts={httpbin.host}")
    assert_socket_blocked(result, passed=1, failed=1)



"""
Common test functions.
"""


def assert_socket_blocked(result, passed=0, skipped=0, failed=1):
    """
    Uses built in methods to test for common failure scenarios.
    Usually we only test for a single failure,
    but sometimes we want to test for multiple conditions,
    so we can pass in the expected counts.
    """
    result.assert_outcomes(passed=passed, skipped=skipped, failed=failed)
    result.stdout.fnmatch_lines(
        "*Socket*Blocked*Error: A test tried to use socket.socket.*"
    )



"""Test module to express odd combinations of configurations."""
from conftest import unix_sockets_only


def test_parametrize_with_socket_enabled_and_allow_hosts(testdir, httpbin):
    """This is a complex test that demonstrates the use of `parametrize`,
    `enable_socket` fixture, allow_hosts CLI flag.

    TODO: This test makes real http calls. httpbin only provides a single IP.
    Is there a better way to express multiple **working** IPs?

    From: https://github.com/miketheman/pytest-socket/issues/56
    """
    testdir.makepyfile(
        f"""
        import pytest
        import requests


        @pytest.mark.parametrize(
            "url",
            [
                "https://google.com",
                "https://amazon.com",
                "https://microsoft.com",
            ],
        )
        def test_domain(url, socket_enabled):
            requests.get(url)

        def test_localhost_works():
            requests.get("{httpbin.url}/")

        def test_remote_not_allowed_fails():
            requests.get("http://172.1.1.1/")
        """
    )
    testdir.makeini(
        f"""
        [pytest]
        addopts = --disable-socket --allow-hosts={httpbin.host}
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(passed=4, failed=1)
    result.stdout.fnmatch_lines(
        "*SocketConnectBlockedError: "
        "A test tried to use socket.socket.connect() with host*"
    )


@unix_sockets_only
def test_combine_unix_and_allow_hosts(testdir, httpbin):
    """Test combination of disable, allow-unix and allow-hosts.

    From https://github.com/miketheman/pytest-socket/issues/78
    """
    testdir.makepyfile(
        f"""
        import socket

        import pytest


        def test_unix_connect():
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            with pytest.raises(FileNotFoundError):
                sock.connect('/tmp/socket.sock')


        def test_inet_connect():
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect(('{httpbin.host}', {httpbin.port}))
        """
    )
    result = testdir.runpytest(
        "--disable-socket", "--allow-unix-socket", f"--allow-hosts={httpbin.host}"
    )
    result.assert_outcomes(passed=2)



import inspect

import pytest

localhost = "127.0.0.1"

connect_code_template = """
    import socket
    import pytest

    {3}
    def {2}():
        socket.socket().connect(('{0}', {1}))
"""

connect_unicode_code_template = """
    import socket
    import pytest

    {3}
    def {2}():
        socket.socket().connect((u'{0}', {1}))
"""

urlopen_code_template = """
    import pytest
    from urllib.request import urlopen

    {3}
    def {2}():
        assert urlopen('http://{0}:{1}/').getcode() == 200
"""

urlopen_hostname_code_template = """
    import pytest
    from urllib.request import urlopen

    {3}
    def {2}():
        # Skip {{1}} as we expect {{0}} to be the full hostname with or without port
        assert urlopen('http://{0}').getcode() == 200
"""


def assert_host_blocked(result, host):
    result.stdout.fnmatch_lines(
        f'*A test tried to use socket.socket.connect() with host "{host}"*'
    )


@pytest.fixture
def assert_connect(httpbin, testdir):
    def assert_socket_connect(should_pass, **kwargs):
        # get the name of the calling function
        test_name = inspect.stack()[1][3]

        mark = ""
        host = kwargs.get("host", httpbin.host)
        cli_arg = kwargs.get("cli_arg", None)
        code_template = kwargs.get("code_template", connect_code_template)
        mark_arg = kwargs.get("mark_arg", None)

        if mark_arg:
            if isinstance(mark_arg, str):
                mark = f'@pytest.mark.allow_hosts("{mark_arg}")'
            elif isinstance(mark_arg, list):
                hosts = '","'.join(mark_arg)
                mark = f'@pytest.mark.allow_hosts(["{hosts}"])'
        code = code_template.format(host, httpbin.port, test_name, mark)
        testdir.makepyfile(code)

        if cli_arg:
            result = testdir.runpytest(f"--allow-hosts={cli_arg}")
        else:
            result = testdir.runpytest()

        if should_pass:
            result.assert_outcomes(1, 0, 0)
        else:
            result.assert_outcomes(0, 0, 1)
            assert_host_blocked(result, host)

        return result

    return assert_socket_connect


def test_help_message(testdir):
    result = testdir.runpytest(
        "--help",
    )
    result.stdout.fnmatch_lines(
        [
            "socket:",
            "*--allow-hosts=ALLOWED_HOSTS_CSV",
            "*Only allow specified hosts through",
            "*socket.socket.connect((host, port)).",
        ]
    )


def test_marker_help_message(testdir):
    result = testdir.runpytest(
        "--markers",
    )
    result.stdout.fnmatch_lines(
        [
            "@pytest.mark.allow_hosts([[]hosts[]]): "
            "Restrict socket connection to defined list of hosts",
        ]
    )


def test_default_connect_enabled(assert_connect):
    assert_connect(True)


def test_single_cli_arg_connect_enabled(assert_connect):
    assert_connect(True, cli_arg=localhost)


def test_single_cli_arg_connect_enabled_localhost_resolved(assert_connect):
    assert_connect(True, cli_arg="localhost")


def test_single_cli_arg_127_0_0_1_hostname_localhost_connect_disabled(assert_connect):
    assert_connect(False, cli_arg=localhost, host="localhost")


def test_single_cli_arg_localhost_hostname_localhost_connect_enabled(assert_connect):
    assert_connect(True, cli_arg="localhost", host="localhost")


def test_single_cli_arg_connect_disabled_hostname_resolved(assert_connect):
    result = assert_connect(
        False,
        cli_arg="localhost",
        host="1.2.3.4",
        code_template=urlopen_hostname_code_template,
    )
    result.stdout.fnmatch_lines(
        '*A test tried to use socket.socket.connect() with host "1.2.3.4" '
        '(allowed: "localhost (127.0.0.1,::1)")*'
    )


def test_single_cli_arg_connect_enabled_hostname_unresolvable(assert_connect):
    assert_connect(False, cli_arg="unresolvable")


def test_single_cli_arg_connect_unicode_enabled(assert_connect):
    assert_connect(True, cli_arg=localhost, code_template=connect_unicode_code_template)


def test_multiple_cli_arg_connect_enabled(assert_connect):
    assert_connect(True, cli_arg=localhost + ",1.2.3.4")


def test_single_mark_arg_connect_enabled(assert_connect):
    assert_connect(True, mark_arg=localhost)


def test_multiple_mark_arg_csv_connect_enabled(assert_connect):
    assert_connect(True, mark_arg=localhost + ",1.2.3.4")


def test_multiple_mark_arg_list_connect_enabled(assert_connect):
    assert_connect(True, mark_arg=[localhost, "1.2.3.4"])


def test_mark_cli_conflict_mark_wins_connect_enabled(assert_connect):
    assert_connect(True, mark_arg=[localhost], cli_arg="1.2.3.4")


def test_single_cli_arg_connect_disabled(assert_connect):
    assert_connect(
        False, cli_arg="1.2.3.4", code_template=connect_unicode_code_template
    )


def test_multiple_cli_arg_connect_disabled(assert_connect):
    assert_connect(False, cli_arg="5.6.7.8,1.2.3.4")


def test_single_mark_arg_connect_disabled(assert_connect):
    assert_connect(False, mark_arg="1.2.3.4")


def test_multiple_mark_arg_csv_connect_disabled(assert_connect):
    assert_connect(False, mark_arg="5.6.7.8,1.2.3.4")


def test_multiple_mark_arg_list_connect_disabled(assert_connect):
    assert_connect(False, mark_arg=["5.6.7.8", "1.2.3.4"])


def test_mark_cli_conflict_mark_wins_connect_disabled(assert_connect):
    assert_connect(False, mark_arg=["1.2.3.4"], cli_arg=localhost)


def test_default_urlopen_succeeds_by_default(assert_connect):
    assert_connect(True, code_template=urlopen_code_template)


def test_single_cli_arg_urlopen_enabled(assert_connect):
    assert_connect(
        True, cli_arg=localhost + ",1.2.3.4", code_template=urlopen_code_template
    )


def test_single_mark_arg_urlopen_enabled(assert_connect):
    assert_connect(
        True, mark_arg=[localhost, "1.2.3.4"], code_template=urlopen_code_template
    )


def test_global_restrict_via_config_fail(testdir):
    testdir.makepyfile(
        """
        import socket

        def test_global_restrict_via_config_fail():
            socket.socket().connect(('127.0.0.1', 80))
        """
    )
    testdir.makeini(
        """
        [pytest]
        addopts = --allow-hosts=2.2.2.2
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(0, 0, 1)
    assert_host_blocked(result, "127.0.0.1")


def test_global_restrict_via_config_pass(testdir, httpbin):
    testdir.makepyfile(
        f"""
        import socket

        def test_global_restrict_via_config_pass():
            socket.socket().connect(('{httpbin.host}', {httpbin.port}))
        """
    )
    testdir.makeini(
        f"""
        [pytest]
        addopts = --allow-hosts={httpbin.host}
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(1, 0, 0)


def test_test_isolation(testdir, httpbin):
    testdir.makepyfile(
        f"""
        import pytest
        import socket

        @pytest.mark.allow_hosts('{httpbin.host}')
        def test_pass():
            socket.socket().connect(('{httpbin.host}', {httpbin.port}))

        @pytest.mark.allow_hosts('2.2.2.2')
        def test_fail():
            socket.socket().connect(('{httpbin.host}', {httpbin.port}))

        def test_pass_2():
            socket.socket().connect(('{httpbin.host}', {httpbin.port}))
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(2, 0, 1)
    assert_host_blocked(result, httpbin.host)


def test_conflicting_cli_vs_marks(testdir, httpbin):
    testdir.makepyfile(
        f"""
        import pytest
        import socket

        @pytest.mark.allow_hosts('{httpbin.host}')
        def test_pass():
            socket.socket().connect(('{httpbin.host}', {httpbin.port}))

        @pytest.mark.allow_hosts('2.2.2.2')
        def test_fail():
            socket.socket().connect(('{httpbin.host}', {httpbin.port}))

        def test_fail_2():
            socket.socket().connect(('2.2.2.2', {httpbin.port}))
        """
    )
    result = testdir.runpytest("--allow-hosts=1.2.3.4")
    result.assert_outcomes(1, 0, 2)
    assert_host_blocked(result, "2.2.2.2")
    assert_host_blocked(result, httpbin.host)



from conftest import unix_sockets_only
from tests.common import assert_socket_blocked


@unix_sockets_only
def test_starlette(testdir):
    testdir.makepyfile(
        """
        from pytest_socket import disable_socket
        from starlette.responses import HTMLResponse
        from starlette.testclient import TestClient


        def pytest_runtest_setup():
            disable_socket()


        async def app(scope, receive, send):
            assert scope['type'] == 'http'
            response = HTMLResponse('<html><body>Hello, world!</body></html>')
            await response(scope, receive, send)


        def test_app():
            client = TestClient(app)
            response = client.get('/')
            assert response.status_code == 200
        """
    )
    result = testdir.runpytest("--disable-socket", "--allow-unix-socket")
    result.assert_outcomes(passed=1, skipped=0, failed=0)


@unix_sockets_only
def test_httpx_fails(testdir):
    testdir.makepyfile(
        """
        import pytest
        import httpx


        @pytest.fixture(autouse=True)
        def anyio_backend():
            return "asyncio"

        async def test_httpx():
            async with httpx.AsyncClient() as client:
                await client.get("http://www.example.com/")
        """
    )
    result = testdir.runpytest("--disable-socket", "--allow-unix-socket")
    assert_socket_blocked(result)
